import boto3

class AWSClient:
  instance = None

  # Singleton to instantiate client only one time between lambdas in warm start
  def __new__(cls, *args, **kwargs):
      if not isinstance(cls.instance, cls):
          cls.instance = object.__new__(cls)
      return cls.instance

  def __init__(self):
    self.s3 = boto3.client("s3")
    
  # Define new AWS Client if not already instantiated
  def getAWSClient(self):
    return self



import os
import sentry_sdk
import logging
from aws_xray_sdk.core import xray_recorder
from aws_xray_sdk.core import patch_all
from .aws_client import AWSClient


# Initialize sentry, logger and AWS X-Ray
def handler_decorator(handler):
  def wrapper(event, context, aws_client: AWSClient):
    
    # Logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # AWS X-Ray patch all traces 
    patch_all()

    # Define AWS Client
    aws_client_class = AWSClient()
    aws_client = aws_client_class.getAWSClient()

    # Sentry
    sentry_sdk.init(
      dsn = os.environ['SENTRY_DSN'] if "SENTRY_DSN" in os.environ else "",
      traces_sample_rate=1.0
    )
    try:
      return handler(event, context, aws_client)
    except Exception as err:
      sentry_sdk.capture_exception(str(err))
      client = sentry_sdk.Hub.current.client
      if client is not None:
        client.flush(timeout=1)
      raise Exception(str(err))

  return wrapper


from setuptools import setup, find_packages

VERSION = '0.0.10'
DESCRIPTION = 'Macai framework for AWS lambda in Python'
LONG_DESCRIPTION = 'Macai framework for AWS lambda in Python'

setup(
    name="framework-python",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author="Macai",
    author_email="developers@macaiapp.com",
    license='MIT',
    packages=find_packages(),
    install_requires=['sentry-sdk>=1.5.7', 'aws-xray-sdk>=2.4.3', 'boto3>=1.21.17'],
    url='https://macaiapp.com',
    classifiers= [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
    ]
)